package intro.userdesk;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.util.tracker.ServiceTracker;

import intro.carrier.service.FlightService;

/**
 * This class implements a bundle that uses a flight
 * service to check for seating availaibility in 
 * either international or domestic carriers. This bundle monitors 
 * the dynamic availability of the flight services. 
 * In other words, if the service it is using
 * becomes unavailable, then it stops using it gracefully, 
 * or if it needs a service and one arrives, then it starts using it
 * automatically. 
 * The bundle uses the first service that it finds and uses the calling 
 * thread of the start() method to read flights from standard input.
 * You can stop checking flights by entering an empty line, but
 * to start checking for flights again you must stop and then restart
 * the bundle.
**/
public class Activator implements BundleActivator
{
    // Bundle's context.
    private BundleContext m_context = null;
    // The service tacker object.
    private ServiceTracker m_tracker = null;

    /**
     * Implements BundleActivator.start(). Crates a service
     * tracker to monitor flight services and starts its "flight
     * checking loop". It will not be able to check any flights until
     * the service tracker find a flight service; any discovered
     * flight service will be automatically used by the client.
     * It reads flights from standard input and checks for their
     * existence in the discovered flight service.
     * (NOTE: It is very bad practice to use the calling thread
     * to perform a lengthy process like this; this is only done
     * for the purpose of the intro.)
     * @param context the framework context for the bundle.
    **/
    public void start(BundleContext context) throws Exception
    {
        m_context = context;

        // Create a service tracker to monitor flight services.
        m_tracker = new ServiceTracker(
            m_context,
            m_context.createFilter(
                "(&(objectClass=" + FlightService.class.getName() + ")" +
                "(Seating=*))"),
            null);
        m_tracker.open();

        try
        {
            System.out.println("Enter a blank line to exit.");
            String destination = "";
            BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

            // Loop endlessly.
            while (true)
            {
                // Ask the user to enter a flight.
                System.out.print("Enter flight: ");
                destination = in.readLine();

                // Get the selected flight service, if available.
                FlightService flight = (FlightService) m_tracker.getService();

                // If the user entered a blank line, then
                // exit the loop.
                if (destination.length() == 0)
                {
                    break;
                }
                // If there is no flight, then say so.
                else if (flight == null)
                {
                    System.out.println("No flight available.");
                }
                // Otherwise print whether the flight is available or not.
                else if (flight.checkAvailability(destination))
                {
                    System.out.println("Flight available to :" + destination);
                }
                else
                {
                    System.out.println("No flight available to :" + destination);
                }
            }
        } catch (Exception ex) { }
    }

    /**
     * Implements BundleActivator.stop(). Does nothing since
     * the framework will automatically unget any used services.
     * @param context the framework context for the bundle.
    **/
    public void stop(BundleContext context)
    {
    }
}
